// implementation.h
//
// Created 1998 by Jouni K Seppnen
//
// $Id: implementation.cc,v 1.7 1999/08/10 07:55:09 jks Exp $
//

#include "implementation.h"
#include "main.h"
#include "distribution.h"

//
// Initialization clauses
//

constantInitializationClause::constantInitializationClause(expr *e, variable *v):
    initializationClause(v), theConstant(e)
{
    if (!e->isConst()) {
	fprintf(stderr, "Warning: Initialization expression for variable %s "
		"is not constant\n", v->giveName().c_str());
    }
};

void
constantInitializationClause::printInitializer()
{
    theConstant->printValue();
}

randomInitializationClause::randomInitializationClause(distributionRef *d, variable *v):
    initializationClause(v), theDistribution(d)
{
    ASSERT(d != 0);
    XlistIterator<expr> i = d->giveParameters()->giveIterator();
    int j;
    for (i.first(), j=1; i.valid(); i++, j++) {
	if (! (*i)->isConst()) {
	    fprintf(stderr, "Warning: Parameter %d of initialization distribution "
		    "for variable %s is not constant\n", j, v->giveName().c_str());
	}
    }
};

void
randomInitializationClause::printInitializer()
{
    theDistribution->printClass();
    fputs(".propose(", c.file);
    theDistribution->printParams();
    fputs(")", c.file);
}

//
// Proposal clauses
//

proposalClause::proposalClause(distributionRef *d, variable *v): 
    theDistribution(d) 
{
    // FIXME: Go through d's parameters, change references to v
    // into some special kind that prints `oldValue' into the code.
    // (??? Is this necessary? --jks 1999-08-02)
};

void
proposalClause::printProposer()
{
    theDistribution->printClass();
    fputs(".propose(", c.file);
    theDistribution->printParams();
    fputs(")", c.file);
}

void
proposalClause::printProposalDensity()
{
    theDistribution->printClass();
    fputs(".proposalDensity(", c.file);
    theDistribution->printParams();
    fputs(", newValue)", c.file);
}


// Local Variables:
// mode: c++
// make-backup-files: t
// c-basic-offset: 4
// End:
