// implementation.h
//
// Created 1998 by Jouni K Seppnen
//
// $Id: implementation.h,v 1.4 1998/12/09 11:31:56 jks Exp $
//

#ifndef IMPLEMENTATION_H
#define IMPLEMENTATION_H

#include "distribution.h"
#include "variable.h"

/** Base class for all implementation clauses. */
class implementationClause {
};

/** Base class for all initialization clauses. */
class initializationClause: public implementationClause {
protected:
    /// The variable being initialized
    variable *theVariable;
public:
    /// Constructor
    initializationClause(variable *v): theVariable(v) {};
    /// Print the value used to initialize the variable
    virtual void printInitializer() = 0;
    /// Resolve references
    virtual void findReferences() = 0;
};

/** Initialization clauses of the form `init foo = bar'. */
class constantInitializationClause: public initializationClause {
    /// The constant initializer expression
    expr *theConstant;
public:
    /// Constructor
    constantInitializationClause(expr *c, variable *v);
    /// Print the value of the constant expression used to initialize the variable
    void printInitializer();
    /// Resolve references: just pass the call to the constant expression
    void findReferences() { theConstant->findReferences(); };
};

/** Initialization clauses of the form `init foo ~ bar(...)'. */
class randomInitializationClause: public initializationClause {
    /// The distribution from which the initial value is drawn
    distributionRef *theDistribution;
public:
    /// Constructor
    randomInitializationClause(distributionRef *d, variable *v);
    /// Print a call to the distribution proposer function
    void printInitializer();
    /// Resolve references: just pass the call to the distribution
    void findReferences() { theDistribution->findReferences(); };
};


/** Proposal clauses */
class proposalClause: public implementationClause {
    /// The distribution used for proposals
    distributionRef *theDistribution;
public:
    /// Constructor
    proposalClause(distributionRef *d, variable *v);
    /// Print a call to the distribution proposer function
    void printProposer();
    /// Print a call to the distribution proposal density function
    void printProposalDensity();
    /// Resolve references: just pass the call to the distribution
    void findReferences() { theDistribution->findReferences(); };
};


#endif // IMPLEMENTATION_H


// Local Variables:
// mode: c++
// make-backup-files: t
// c-basic-offset: 4
// End:
