// mt-rng.h
//
// This file created on August 20, 1998 by Jouni Seppnen, based on 
// Shawn Cokus's file dated March 8, 1998; see
// http://www.math.keio.ac.jp/~matumoto/cokus.c
//
// @(#)$Id: mt-rng.cc,v 1.2 1998/08/31 20:01:11 jks Exp $
//
// In the following, "this library" refers to the files mt-rng.h and
// mt-rng.cc.
//
// This library is free software; you can redistribute it and/or modify it
// under the terms of the GNU Library General Public License as published by
// the Free Software Foundation (either version 2 of the License or, at your
// option, any later version).  This library is distributed in the hope that
// it will be useful, but WITHOUT ANY WARRANTY, without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
// the GNU Library General Public License for more details.  You should have
// received a copy of the GNU Library General Public License along with this
// library; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307, USA.
//
// Copyright notices from the original version:
//
// This version is a recode by Shawn Cokus (Cokus@math.washington.edu)
// on March 8, 1998 of a version by Takuji Nishimura (who had
// suggestions from Topher Cooper and Marc Rieffel in July-August
// 1997).
//
// The code as Shawn received it included the following notice:
//
//   Copyright (C) 1997 Makoto Matsumoto and Takuji Nishimura.  When
//   you use this, send an e-mail to <matumoto@math.keio.ac.jp> with
//   an appropriate reference to your work.
//
// It would be nice to CC: <Cokus@math.washington.edu> when you write.
//

#include "mt-rng.h"

void MT_RandomNumberGenerator::seedMT(unsigned long seed)
{
    register uint32 x = (seed | 1U) & 0xFFFFFFFFU, *s = state;
    register int    j;

    for(left=0, *s++=x, j=N; --j;
        *s++ = (x*=69069U) & 0xFFFFFFFFU);
}

MT_RandomNumberGenerator::uint32 MT_RandomNumberGenerator::reloadMT()
{
    register uint32 *p0=state, *p2=state+2, *pM=state+M, s0, s1;
    register int    j;

    if(left < -1)
        seedMT(4357U);

    left=N-1, next=state+1;

    for(s0=state[0], s1=state[1], j=N-M+1; --j; s0=s1, s1=*p2++)
        *p0++ = *pM++ ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);

    for(pM=state, j=M; --j; s0=s1, s1=*p2++)
        *p0++ = *pM++ ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);

    s1=state[0], *p0 = *pM ^ (mixBits(s0, s1) >> 1) ^ (loBit(s1) ? K : 0U);
    s1 ^= (s1 >> 11);
    s1 ^= (s1 <<  7) & 0x9D2C5680U;
    s1 ^= (s1 << 15) & 0xEFC60000U;
    return(s1 ^ (s1 >> 18));
}

//------------------------------------------------
// Local Variables:				//
// mode: c++					//
// make-backup-files: t				//
// c-basic-offset: 4                            //
// End:						//
//------------------------------------------------
