// genRecords.cc
//
// Created 1998 by Karri-Pekka Laakso
//
// $Id: genRecords.cc,v 1.18 1999/08/10 12:19:22 jks Exp $
//

#include "genRecords.h"

// A map is created for every record for the initialisation. This is
// just the pointer instantiation, because the maps are allocated
// dynamically in the records' constructors.

// FOREACH REC (RECORD) 
record_$REC.NAME::instanceMapType *record_$REC.NAME::instanceMap = 0;
// ENDFOR

// In addition to the map, a list is created for all record types.
// This is needed for the implementation of aggregate functions.
// This list should eventually get replaced by the map above.

// FOREACH REC (RECORD)
vector<record_$REC.NAME*> *record_$REC.NAME::instances = new vector<record_$REC.NAME*>;
// ENDFOR

// Record class implementation

// FOREACH REC (RECORD)

//----------------------------------------
// record_$REC.NAME
//----------------------------------------


//---------------------------------------------------------------------
// record_$REC.NAME::record_$REC.NAME
//  The constructor for record_$REC.NAME.
//
// 
// --------------------------------------------------------------------

record_$REC.NAME::record_$REC.NAME(const string &n) : $REC.BASE(n)
{
    if (instanceMap == 0) {
	instanceMap = new instanceMapType;
	DEBUG('r', "Created map for $REC.NAME\n");
    }

    pair<string, record_$REC.NAME*> p(name, this);
    pair<instanceMapType::iterator, bool> result = instanceMap->insert(p);
    if ( ! result.second) {
	fprintf(stderr,
		"The $REC.NAME record \"%s\" was defined more than once in the data\n",
		name.c_str());
	exit(1);
    }

    instances->push_back(this);

    DEBUG('r', "Creating $REC.NAME record %s\n", name.c_str());

// FOREACH REF (REC_REFFIELD)
    $REF.VNAMERef = 0;
// ENDFOR    

// FOREACH VAR (REC_VARFIELD) 
    $VAR.NAME = 0;
// ENDFOR
}

//----------------------------------------------------------------------
// record_$REC.NAME::createVariables
//----------------------------------------------------------------------

void
record_$REC.NAME::createVariables()
{
    DEBUG('I', "Creating variables for $REC.NAME %s:\n", name.c_str());
// FOREACH VAR (REC_VARFIELD) 
    DEBUG('I', "    $VAR.NAME\n");
    if ($VAR.NAME != 0) {
	fprintf(stderr, "Internal error: variable initialisation all fouled up.\n"
		"Please mention \"%s.%s\" when reporting the bug.\n",
		"$REC.NAME", "$VAR.NAME");
	exit(1);
    } else {
	$VAR.CREATE
    }
// ENDFOR
}

//---------------------------------------------------------------------
// record_$REC.NAME::readInput
//  The data reading function for record_$REC.NAME
//
// 
// --------------------------------------------------------------------

bool
record_$REC.NAME::readInput(scanner &sc, bool init)
{
    if (!sc.ask("$REC.NAME")) {
	return false;
    }
  
    DEBUG('r', "Reading $REC.NAME record\n");

    // First try to read a name for this record
    char *name = 0;
    if (sc.next(false) != 0) {
	name = new char[strlen(sc.next(false))+1];
	sc.getId(name);
    }

    // Then come the possible references to other records
// FOREACH REF (REC_REFFIELD)
    char *$REF.VNAMERef = new char[scanner::MAXID+1];
    $REF.VNAMERef[0] = '\0';

// ENDFOR    

    if (sc.ask("[")) {
// FOREACH REF (REC_REFFIELD)
	if ( ! sc.getId($REF.VNAMERef)) {
	    fprintf(stderr,
		    "%s:%d: missing reference definition for \"$REF.VNAME\" reading record $REC.NAME\n",
		    sc.fileName(), sc.line());
	    exit(1);
	}
	sc.ask(","); // The "," is optional

// ENDFOR
	if ( ! sc.ask("]")) {
	    fprintf(stderr,
		    "%s:%d: missing terminal symbol ']' reading record $REC.NAME\n",
		    sc.fileName(), sc.line());
	    exit(1);
	}
    }

    // If there are references, check that every reference got defined
// FOREACH REF (REC_REFFIELD)
    if ($REF.VNAMERef[0] == '\0') {
	fprintf(stderr,
		"%s:%d: missing reference definition for \"$REF.VNAME\" reading record $REC.NAME\n",
	       sc.fileName(), sc.line());
	exit(1);
    }
// ENDFOR

    if (name == 0) {
	name = new char[1+
// FOREACH REF (REC_REFFIELD)
		       strlen($REF.VNAMERef)+1+
// ENDFOR    
		       1];
	name[0] = '\0';
// FOREACH REF (REC_REFFIELD)
	strcat(name, $REF.VNAMERef);
	strcat(name, "-");
// ENDFOR    
	if (name[0] == '\0') {
	    fprintf(stderr,
		    "%s:%d: record $REC.NAME must have a name\n",
		    sc.fileName(), sc.line());
	    exit(1);
	}
	
	name[strlen(name)-1] = '\0';
    }
	
    record_$REC.NAME *r;
    if ( !init) {
	// Data: create the record instance
	r = new record_$REC.NAME(name);
	r->createVariables();
// FOREACH REF (REC_REFFIELD)
	r->$REF.VNAMERef = $REF.VNAMERef;
// ENDFOR    
    } else {
	// Init: find the record instance
	r = findReference(name);
	if (r == 0) {
	    fprintf(stderr,
		    "An instance of record $REC.NAME named \"%s\" was not found\n",
		    name);
	    exit(1);
	}	

	// Clear some memory
// FOREACH REF (REC_REFFIELD)
	delete $REF.VNAMERef;
// ENDFOR    
    }
	
    // Finally read the other fields
    if (sc.ask("{"))
	while( ! sc.ask("}")) {
	    if (0
// FOREACH VAR (REC_VARFIELD)
		|| r->$VAR.NAME->askValue(sc, "$VAR.VNAME", init)
// ENDFOR		    
		)
	    {
		// ok, do nothing
	    } else {
		fprintf(stderr,
			"%s:%d: unknown variable \"%s\" in record $REC.NAME\n",
		        sc.fileName(), sc.line(), sc.next());
		exit(1);
	    }

	    sc.ask(","); // The "," is optional
	};

    return true;
}


//---------------------------------------------------------------------
// record_$REC.NAME::initLinks
//  Initialize the links of the record for simulation.
//
// Searches and initializes the actual records by the reference names.
// --------------------------------------------------------------------

void
record_$REC.NAME::initLinks()
{
    DEBUG('I', "Initialising links for record_$REC.NAME \"%s\"\n", name.c_str());
// FOREACH REF (REC_REFFIELD)
    $REF.NAME = record_$REF.RECNAME::findReference($REF.VNAMERef);
    if ($REF.NAME == 0) {
	fprintf(stderr,
		"An instance named \"%s\" for reference '$REF.VNAME' in record %s was not found\n",
		$REF.VNAMERef, name.c_str());
	exit(1);
    }	
// ENDFOR    
}


//---------------------------------------------------------------------
// record_$REC.NAME::initialize
//  Initialize the record for simulation.
//
// Initializes the dependencies and values for each variable field
// in the record. 
// --------------------------------------------------------------------

void
record_$REC.NAME::initialize()
{
    DEBUG('I', "Initialising record_$REC.NAME \"%s\"\n", name.c_str());
// FOREACH VAR (REC_VARFIELD)
$VAR.DEPENDENCIES
// ENDFOR    

// FOREACH VAR (REC_VARFIELD)
    $VAR.INIT
// ENDFOR    
}


//---------------------------------------------------------------------
// record_$REC.NAME::findReference
//  Find a record instance by name reference.
//
// --------------------------------------------------------------------

record_$REC.NAME *
record_$REC.NAME::findReference(char *ref)
{
    instanceMapType::iterator pos = instanceMap->find(ref);
    if (pos != instanceMap->end()) {
	return pos->second;
    } else {
	return 0;
    }
}

// Aggregate functions

// FOREACH VAR (REC_VARFIELD)

$VAR.AG_IMPL_CODE

// ENDFOR

// ENDFOR 


// Local Variables:
// mode: c++    
// make-backup-files: t
// c-basic-offset: 4
// case-replace: nil
// End:
